@@ -90,6 +90,7 @@ enum iGmLinkFlag {
     query_GmLinkFlag              = iBit(14), /* Gopher query link */
     iconFromLabel_GmLinkFlag      = iBit(15), /* use an Emoji/special character from label */
     isOpen_GmLinkFlag             = iBit(16), /* currently open in a tab */
+    fontpackFileExtension_GmLinkFlag = iBit(17),
 };
 
 iLocalDef enum iGmLinkScheme scheme_GmLinkFlag(int flags) {
@@ -119,34 +120,31 @@ enum iGmRunFlags {
     decoration_GmRunFlag  = iBit(1), /* not part of the source */
     startOfLine_GmRunFlag = iBit(2),
     endOfLine_GmRunFlag   = iBit(3),
-    siteBanner_GmRunFlag  = iBit(4), /* area reserved for the site banner */
+//    siteBanner_GmRunFlag  = iBit(4), /* area reserved for the site banner */
     quoteBorder_GmRunFlag = iBit(5),
     wide_GmRunFlag        = iBit(6), /* horizontally scrollable */
-    footer_GmRunFlag      = iBit(7),
+//    footer_GmRunFlag      = iBit(7),
     altText_GmRunFlag     = iBit(8),
 };
 
-enum iGmRunMediaType {
-    none_GmRunMediaType,
-    image_GmRunMediaType,
-    audio_GmRunMediaType,
-    download_GmRunMediaType,
-};
-
+/* This structure is tightly packed because GmDocuments are mostly composed of
+   a large number of GmRuns. */
 struct Impl_GmRun {
     iRangecc  text;
     iRect     bounds;    /* used for hit testing, may extend to edges */
     iRect     visBounds; /* actual visual bounds */
     struct {
-        uint16_t   color : 8;
-        uint16_t   font  : 7;
-        uint16_t   isRTL : 1;
-    } textParams;
-    uint8_t   flags;
-    uint8_t   mediaType;
-    uint16_t  preId;     /* preformatted block ID (sequential) */
-    iGmLinkId linkId;    /* zero for non-links */
-    uint16_t  mediaId;   /* zero if not an image */
+        uint32_t linkId    : 16; /* GmLinkId; zero for non-links */
+        uint32_t flags     : 8; /* GmRunFlags */
+        uint32_t isRTL     : 1;
+        uint32_t color     : 7; /* see max_ColorId */
+
+        uint32_t font      : 14;
+        uint32_t mediaType : 3; /* note: max_MediaType means preformatted block */
+        uint32_t mediaId   : 11; /* zero if not an image */
+        uint32_t lineType  : 3;
+        uint32_t isLede    : 1;
+    };
 };
 
 iDeclareType(GmRunRange)
@@ -156,16 +154,35 @@ struct Impl_GmRunRange {
     const iGmRun *end;
 };
 
-iRangecc    findLoc_GmRun   (const iGmRun *, iInt2 pos);
+iLocalDef iBool isMedia_GmRun(const iGmRun *d) {
+    return d->mediaType > 0 && d->mediaType < max_MediaType;
+}
+iLocalDef iMediaId mediaId_GmRun(const iGmRun *d) {
+    if (d->mediaType < max_MediaType) {
+        return (iMediaId){ .type = d->mediaType, .id = d->mediaId };
+    }
+    return iInvalidMediaId;
+}
+iLocalDef uint32_t preId_GmRun(const iGmRun *d) {
+    return d->mediaType == max_MediaType ? d->mediaId : 0;
+}
+
+iRangecc    findLoc_GmRun           (const iGmRun *, iInt2 pos);
 
 iDeclareClass(GmDocument)
 iDeclareObjectConstruction(GmDocument)
+    
+enum iGmDocumentWarning {
+    ansiEscapes_GmDocumentWarning   = iBit(1),
+    missingGlyphs_GmDocumentWarning = iBit(2),
+};
 
+/*
 enum iGmDocumentBanner {
     none_GmDocumentBanner,
     siteDomain_GmDocumentBanner,
     certificateWarning_GmDocumentBanner,
-};
+};*/
 
 enum iGmDocumentUpdate {
     partial_GmDocumentUpdate, /* appending more content */
@@ -174,7 +191,7 @@ enum iGmDocumentUpdate {
 
 void    setThemeSeed_GmDocument (iGmDocument *, const iBlock *seed);
 void    setFormat_GmDocument    (iGmDocument *, enum iSourceFormat format);
-void    setBanner_GmDocument    (iGmDocument *, enum iGmDocumentBanner type);
+//void    setBanner_GmDocument    (iGmDocument *, enum iGmDocumentBanner type);
 void    setWidth_GmDocument     (iGmDocument *, int width, int outsideMargin);
 void    redoLayout_GmDocument   (iGmDocument *);
 iBool   updateOpenURLs_GmDocument(iGmDocument *);
@@ -202,18 +219,23 @@ const iGmRun *  renderProgressive_GmDocument(const iGmDocument *d, const iGmRun
                                              iRangei visRangeY, iGmDocumentRenderFunc render,
                                              void *context);
 iInt2           size_GmDocument             (const iGmDocument *);
-const iGmRun *  siteBanner_GmDocument       (const iGmDocument *);
-iBool           hasSiteBanner_GmDocument    (const iGmDocument *);
-enum iGmDocumentBanner bannerType_GmDocument(const iGmDocument *);
-const iString * bannerText_GmDocument       (const iGmDocument *);
+//const iGmRun *  siteBanner_GmDocument       (const iGmDocument *);
+//iBool           hasSiteBanner_GmDocument    (const iGmDocument *);
+//enum iGmDocumentBanner bannerType_GmDocument(const iGmDocument *);
+//const iString * bannerText_GmDocument       (const iGmDocument *);
 const iArray *  headings_GmDocument         (const iGmDocument *); /* array of GmHeadings */
 const iString * source_GmDocument           (const iGmDocument *);
 size_t          memorySize_GmDocument       (const iGmDocument *); /* bytes */
+int             warnings_GmDocument         (const iGmDocument *);
 
 iRangecc        findText_GmDocument                 (const iGmDocument *, const iString *text, const char *start);
 iRangecc        findTextBefore_GmDocument           (const iGmDocument *, const iString *text, const char *before);
 iGmRunRange     findPreformattedRange_GmDocument    (const iGmDocument *, const iGmRun *run);
 
+int             ansiEscapes_GmDocument              (const iGmDocument *);
+void            runBaseAttributes_GmDocument        (const iGmDocument *, const iGmRun *run,
+                                                     int *fontId_out, int *colorId_out);
+
 enum iGmLinkPart {
     icon_GmLinkPart,
     text_GmLinkPart,
@@ -240,3 +262,4 @@ const iGmPreMeta *preMeta_GmDocument    (const iGmDocument *, uint16_t preId);
 iInt2           preRunMargin_GmDocument (const iGmDocument *, uint16_t preId);
 iBool           preIsFolded_GmDocument  (const iGmDocument *, uint16_t preId);
 iBool           preHasAltText_GmDocument(const iGmDocument *, uint16_t preId);
+
