@@ -29,6 +29,8 @@ iDefineTypeConstruction(VisBuf)
 void init_VisBuf(iVisBuf *d) {
     d->texSize = zero_I2();
     iZap(d->buffers);
+    iZap(d->vis);
+    d->bufferInvalidated = NULL;
 }
 
 void deinit_VisBuf(iVisBuf *d) {
@@ -36,9 +38,14 @@ void deinit_VisBuf(iVisBuf *d) {
 }
 
 void invalidate_VisBuf(iVisBuf *d) {
+    int origin = iMax(0, d->vis.start - d->texSize.y);
     iForIndices(i, d->buffers) {
-        d->buffers[i].origin = i * d->texSize.y;
+        d->buffers[i].origin = origin;
+        origin += d->texSize.y;
         iZap(d->buffers[i].validRange);
+        if (d->bufferInvalidated) {
+            d->bufferInvalidated(d, i);
+        }
     }
 }
 
@@ -51,16 +58,16 @@ void alloc_VisBuf(iVisBuf *d, const iInt2 size, int granularity) {
             if (tex->texture) {
                 SDL_DestroyTexture(tex->texture);
             }
+            SDL_Renderer *rend = renderer_Window(get_Window());
             tex->texture =
-                SDL_CreateTexture(renderer_Window(get_Window()),
+                SDL_CreateTexture(rend,
                                   SDL_PIXELFORMAT_RGBA8888,
                                   SDL_TEXTUREACCESS_STATIC | SDL_TEXTUREACCESS_TARGET,
                                   texSize.x,
                                   texSize.y);
             SDL_SetTextureBlendMode(tex->texture, SDL_BLENDMODE_NONE);
-            tex->origin = i * texSize.y;
-            iZap(tex->validRange);
         }
+        invalidate_VisBuf(d);
     }
 }
 
@@ -72,55 +79,72 @@ void dealloc_VisBuf(iVisBuf *d) {
     }
 }
 
-void reposition_VisBuf(iVisBuf *d, const iRangei vis) {
+static void roll_VisBuf_(iVisBuf *d, int dir) {
+    const size_t lastPos = iElemCount(d->buffers) - 1;
+    if (dir < 0) {
+        /* Last buffer is moved to the beginning. */
+        SDL_Texture *last = d->buffers[lastPos].texture;
+        void *       user = d->buffers[lastPos].user;
+        memmove(d->buffers + 1, d->buffers, sizeof(iVisBufTexture) * lastPos);
+        d->buffers[0].texture = last;
+        d->buffers[0].user    = user;
+        d->buffers[0].origin  = d->buffers[1].origin - d->texSize.y;
+        iZap(d->buffers[0].validRange);
+        if (d->bufferInvalidated) {
+            d->bufferInvalidated(d, 0);
+        }
+    }
+    else {
+        /* First buffer is moved to the end. */
+        SDL_Texture *first = d->buffers[0].texture;
+        void *       user  = d->buffers[0].user;
+        memmove(d->buffers, d->buffers + 1, sizeof(iVisBufTexture) * lastPos);
+        d->buffers[lastPos].texture = first;
+        d->buffers[lastPos].user    = user;
+        d->buffers[lastPos].origin  = d->buffers[lastPos - 1].origin + d->texSize.y;
+        iZap(d->buffers[lastPos].validRange);
+        if (d->bufferInvalidated) {
+            d->bufferInvalidated(d, lastPos);
+        }
+    }
+}
+
+iBool reposition_VisBuf(iVisBuf *d, const iRangei vis) {
+    if (equal_Rangei(vis, d->vis)) {
+        return iFalse;
+    }
+    const int moveDir = vis.end > d->vis.end ? +1 : -1;
     d->vis = vis;
-    iRangei good = { 0, 0 };
-    size_t avail[iElemCount(d->buffers)], numAvail = 0;
-    /* Check which buffers are available for reuse. */ {
-        iForIndices(i, d->buffers) {
-            iVisBufTexture *buf    = d->buffers + i;
-            const iRangei   region = { buf->origin, buf->origin + d->texSize.y };
-            if (isEmpty_Rangei(buf->validRange) ||
-                buf->validRange.start >= vis.end || buf->validRange.end <= vis.start) {
-                avail[numAvail++] = i;
-                iZap(buf->validRange);
-            }
-            else {
-                good = union_Rangei(good, region);
-            }
-        }
-    }
     iBool wasChanged = iFalse;
-    iBool doReset    = (numAvail == iElemCount(d->buffers));
-    /* Try to extend to cover the visible range. */
-    while (!doReset && vis.start < good.start) {
-        if (numAvail == 0) {
-            doReset = iTrue;
-            break;
-        }
-        good.start -= d->texSize.y;
-        d->buffers[avail[--numAvail]].origin = good.start;
+    const size_t lastPos = iElemCount(d->buffers) - 1;
+    if (d->buffers[0].origin > vis.end || d->buffers[lastPos].origin + d->texSize.y <= vis.start) {
+        /* All buffers outside the visible region. */
+        invalidate_VisBuf(d);
         wasChanged = iTrue;
     }
-    while (!doReset && vis.end > good.end) {
-        if (numAvail == 0) {
-            doReset = iTrue;
-            break;
+    else {
+        /* Check for mandatory rolls. */
+        while (d->buffers[0].origin > vis.start) {
+            roll_VisBuf_(d, -1);
+            wasChanged = iTrue;
         }
-        d->buffers[avail[--numAvail]].origin = good.end;
-        good.end += d->texSize.y;
-        wasChanged = iTrue;
-    }
-    if (doReset) {
-//        puts("VisBuf reset!");
-//        fflush(stdout);
-        wasChanged = iTrue;
-        int pos = -1;
-        iForIndices(i, d->buffers) {
-            iZap(d->buffers[i].validRange);
-            d->buffers[i].origin = vis.start + pos++ * d->texSize.y;
+        if (!wasChanged) {
+            while (d->buffers[lastPos].origin + d->texSize.y < vis.end) {
+                roll_VisBuf_(d, +1);
+                wasChanged = iTrue;
+            }
+        }
+        /* Scroll-direction dependent optional rolls, with a bit of overscroll allowed. */
+        if (moveDir > 0 && d->buffers[0].origin + d->texSize.y + d->texSize.y / 4 < vis.start) {
+            roll_VisBuf_(d, +1);
+            wasChanged = iTrue;
+        }
+        else if (moveDir < 0 && d->buffers[lastPos].origin - d->texSize.y / 4 > vis.end) {
+            roll_VisBuf_(d, -1);
+            wasChanged = iTrue;
         }
     }
+    iUnused(wasChanged);
 #if 0
     if (wasChanged) {
         printf("\nVISIBLE RANGE: %d ... %d\n", vis.start, vis.end);
@@ -134,6 +158,33 @@ void reposition_VisBuf(iVisBuf *d, const iRangei vis) {
         fflush(stdout);
     }
 #endif
+#if !defined (NDEBUG)
+    /* Buffers must not overlap. */
+    iForIndices(m, d->buffers) {
+        const iRangei M = { d->buffers[m].origin, d->buffers[m].origin + d->texSize.y };
+        iForIndices(n, d->buffers) {
+            if (m == n) continue;
+            const iRangei N = { d->buffers[n].origin, d->buffers[n].origin + d->texSize.y };
+            const iRangei is = intersect_Rangei(M, N);
+            if (size_Range(&is) != 0) {
+                printf("buffers %zu (%i) and %zu (%i) overlap\n",
+                       m, M.start, n, N.start);
+                fflush(stdout);
+            }
+            iAssert(size_Range(&is) == 0);
+        }
+    }
+#endif
+    return iTrue; /* at least the visible range changed */
+}
+
+iRangei allocRange_VisBuf(const iVisBuf *d) {
+    return (iRangei){ d->buffers[0].origin,
+                      d->buffers[iElemCount(d->buffers) - 1].origin + d->texSize.y };
+}
+
+iRangei bufferRange_VisBuf(const iVisBuf *d, size_t index) {
+    return (iRangei){ d->buffers[index].origin, d->buffers[index].origin + d->texSize.y };
 }
 
 void invalidRanges_VisBuf(const iVisBuf *d, const iRangei full, iRangei *out_invalidRanges) {
@@ -160,7 +211,7 @@ void validate_VisBuf(iVisBuf *d) {
 
 //#define DEBUG_SCALE 0.5f
 
-void draw_VisBuf(const iVisBuf *d, iInt2 topLeft) {
+void draw_VisBuf(const iVisBuf *d, const iInt2 topLeft, const iRangei yClipBounds) {
     SDL_Renderer *render = renderer_Window(get_Window());
     iForIndices(i, d->buffers) {
         const iVisBufTexture *buf = d->buffers + i;
@@ -168,6 +219,11 @@ void draw_VisBuf(const iVisBuf *d, iInt2 topLeft) {
                          topLeft.y + buf->origin,
                          d->texSize.x,
                          d->texSize.y };
+        if (dst.y >= yClipBounds.end || dst.y + dst.h < yClipBounds.start) {
+#if !defined (DEBUG_SCALE)
+            continue; /* Outside the clipping area. */
+#endif
+        }
 #if defined (DEBUG_SCALE)
         dst.w *= DEBUG_SCALE;
         dst.h *= DEBUG_SCALE;
@@ -177,5 +233,20 @@ void draw_VisBuf(const iVisBuf *d, iInt2 topLeft) {
         dst.y += get_Window()->root->rect.size.y / 4;
 #endif
         SDL_RenderCopy(render, buf->texture, NULL, &dst);
+#if defined (DEBUG_SCALE)
+        SDL_SetRenderDrawColor(render, 0, 0, 255, 255);
+        SDL_RenderDrawRect(render, &dst);
+#endif
     }
+#if defined (DEBUG_SCALE)
+    SDL_Rect dst = { topLeft.x, yClipBounds.start, d->texSize.x, 2 * d->texSize.y };
+    dst.w *= DEBUG_SCALE;
+    dst.h *= DEBUG_SCALE;
+    dst.x *= DEBUG_SCALE;
+    dst.y *= DEBUG_SCALE;
+    dst.x += get_Window()->root->rect.size.x / 4;
+    dst.y += get_Window()->root->rect.size.y / 4;
+    SDL_SetRenderDrawColor(render, 255, 255, 255, 255);
+    SDL_RenderDrawRect(render, &dst);
+#endif
 }
