@@ -973,6 +973,18 @@ iLocalDef iBool isMouseEvent_(const SDL_Event *ev) {
             ev->type == SDL_MOUSEBUTTONUP || ev->type == SDL_MOUSEBUTTONDOWN);
 }
 
+iLocalDef iBool isHidden_Widget_(const iWidget *d) {
+    if (d->flags & visibleOnParentHover_WidgetFlag &&
+        (isHover_Widget(d) || isHover_Widget(d->parent))) {
+        return iFalse;
+    }
+    return (d->flags & hidden_WidgetFlag) != 0;
+}
+
+iLocalDef iBool isDrawn_Widget_(const iWidget *d) {
+    return !isHidden_Widget_(d) || d->flags & visualOffset_WidgetFlag;
+}
+
 static iBool filterEvent_Widget_(const iWidget *d, const SDL_Event *ev) {
     if (d->flags & destroyPending_WidgetFlag) {
         return iFalse; /* no more events handled */
@@ -983,7 +995,7 @@ static iBool filterEvent_Widget_(const iWidget *d, const SDL_Event *ev) {
                                              d->flags & disabledWhenHidden_WidgetFlag)) {
         if (isKey || isMouse) return iFalse;
     }
-    if (d->flags & hidden_WidgetFlag) {
+    if (isHidden_Widget_(d)) {
         if (isMouse) return iFalse;
     }
     return iTrue;
@@ -1030,9 +1042,9 @@ iBool dispatchEvent_Widget(iWidget *d, const SDL_Event *ev) {
         }
     }
     else if (ev->type == SDL_MOUSEMOTION &&
-             ev->motion.windowID == SDL_GetWindowID(window_Widget(d)->win) &&
+             ev->motion.windowID == id_Window(window_Widget(d)) &&
              (!window_Widget(d)->hover || hasParent_Widget(d, window_Widget(d)->hover)) &&
-             flags_Widget(d) & hover_WidgetFlag && ~flags_Widget(d) & hidden_WidgetFlag &&
+             flags_Widget(d) & hover_WidgetFlag && !isHidden_Widget_(d) &&
              ~flags_Widget(d) & disabled_WidgetFlag) {
         if (contains_Widget(d, init_I2(ev->motion.x, ev->motion.y))) {
             setHover_Widget(d);
@@ -1282,10 +1294,6 @@ int backgroundFadeColor_Widget(void) {
     }
 }
 
-iLocalDef iBool isDrawn_Widget_(const iWidget *d) {
-    return ~d->flags & hidden_WidgetFlag || d->flags & visualOffset_WidgetFlag;
-}
-
 void drawLayerEffects_Widget(const iWidget *d) {
     /* Layered effects are not buffered, so they are drawn here separately. */
     iAssert(isDrawn_Widget_(d));
@@ -1407,7 +1415,8 @@ void drawBackground_Widget(const iWidget *d) {
             fillRect_Paint(&p, rect, d->bgColor);
         }
         if (d->frameColor >= 0 && ~d->flags & frameless_WidgetFlag) {
-            drawRectThickness_Paint(&p, rect, gap_UI / 4, d->frameColor);
+            drawRectThickness_Paint(&p, adjusted_Rect(rect, zero_I2(), neg_I2(one_I2())),
+                                    gap_UI / 4, d->frameColor);
         }
     }
     if (d->flags & (borderTop_WidgetFlag | borderBottom_WidgetFlag)) {
@@ -1699,7 +1708,7 @@ iAny *child_Widget(iWidget *d, size_t index) {
     return NULL;
 }
 
-size_t childIndex_Widget(const iWidget *d, const iAnyObject *child) {
+size_t indexOfChild_Widget(const iWidget *d, const iAnyObject *child) {
     size_t index = 0;
     iConstForEach(ObjectList, i, d->children) {
         if (i.object == child) {
@@ -1711,7 +1720,7 @@ size_t childIndex_Widget(const iWidget *d, const iAnyObject *child) {
 }
 
 iAny *hitChild_Widget(const iWidget *d, iInt2 coord) {
-    if (d->flags & hidden_WidgetFlag) {
+    if (isHidden_Widget_(d)) {
         return NULL;
     }
     /* Check for on-top widgets first. */
